function curve = nrbcirc(radius,cntr,sang,eang)
// NURB representation of a circle or circular arc in the x,y plane.
//
//   curve = nrbcirc([radius,cntr,sang,eang])
//
// Inputs:
//
//   radius - radius (default: 1)
//   cntr   - coordinate of the centre (default: 0,0,0)
//   sang   - start angle relative to the origin (default: 0)
//   eang   - end angle relative to the origin (default: 2pi)
//
// Outputs:
//
//   curve  - NURB representation of circle
//
// Notes:
//
//   The arc is in the x-y plane.
//   The angle is defined as anti-clockwise positive.
//   The arc is made up from 1 to 4 arc segments each
//   with a sweep angle less than or equal to pi/2.
//

[nlhs,nrhs] = argn(0);

if nrhs < 1
  radius = 1;
end

if nrhs < 2
  cntr = [];
end
   
if nrhs < 4
  sang = 0;
  eang = 2*%pi;
end

sweep = eang - sang;       // sweep angle of arc
if sweep < 0 then
  sweep = 2*%pi + sweep;
end
     
if abs(sweep) <= %pi/2
  narcs = 1;                // number of arc segments
  knots = [0 0 0 1 1 1];
elseif abs(sweep) <= %pi
  narcs = 2;
  knots = [0 0 0 0.5 0.5 1 1 1];
elseif abs(sweep) <= 3*%pi/2
  narcs = 3;
  knots = [0 0 0 1/3 1/3 2/3 2/3 1 1 1];
else
  narcs = 4;
  knots = [0 0 0 0.25 0.25 0.5 0.5 0.75 0.75 1 1 1];
end

dsweep = sweep/(2*narcs);     // arc segment sweep angle/2

// determine middle control point and weight
wm = cos(dsweep);
x  = radius*cos(dsweep);
y  = radius*sin(dsweep);
xm = x+y*tan(dsweep);

// arc segment control points
ctrlpt = [ x wm*xm x;    // w*x - coordinate
          -y 0     y;    // w*y - coordinate
           0 0     0;    // w*z - coordinate
           1 wm    1];   // w   - coordinate
      
// build up complete arc from rotated segments
coefs = zeros(4,2*narcs+1);   // nurb control points of arc
xx = vecrotz(sang + dsweep);
coefs(:,1:3) = xx*ctrlpt;     // rotate to start angle
xx = vecrotz(2*dsweep);
for n = 2:narcs
   m = 2*n+[0 1];
   coefs(:,m) = xx*coefs(:,m-2);
end

// translate arc if necessary
if cntr ~= []
  xx = vectrans(cntr);
  coefs = xx*coefs;
end

curve = nrbmak(coefs,knots);
endfunction

