function srf = nrbcoons(u1, u2, v1, v2)
// Construction of a bilinearly blended Coons patch.
//
// INPUTS:
//
//   u1, u2, v1 ,v2 - four NURBS curves defining boundaries.
//
// OUTPUT:
//
//   srf - Coons patch.
//
// NOTES:
//
// The orientation of the four NURBS boundary curves.
//
//         ^ v direction
//         |
//         |     u2
//         ------->--------
//         |              |
//         |              |
//      v1 ^   Surface    ^ v2
//         |              |
//         |              |
//         ------->-----------> u direction
//               u1
//

[nlhs,nrhs] = argn(0);

if nrhs ~= 4
  error('Incorrect number of input arguments');
end

r1 = nrbruled(u1, u2);
r2 = nrbtranspose(nrbruled(v1, v2));
t  = nrb4surf(u1('cntrl')(:,1), u1('cntrl')(:,$), u2('cntrl')(:,1), ...
              u2('cntrl')(:,$));

// raise all surfaces to a common degree  
du = max(r1('degree')(1), r2('degree')(1), t('degree')(1));
dv = max(r1('degree')(2), r2('degree')(2), t('degree')(2));
r1 = nrbdegelev(r1, du - r1('degree')(1), dv - r1('degree')(2));
r2 = nrbdegelev(r2, du - r2('degree')(1), dv - r2('degree')(2));
t  = nrbdegelev(t,  du - t('degree')(1),  dv - t('degree')(2));

// merge the knot vectors, to obtain a common knot vector

// U knots
k1 = r1('uknots');
k2 = r2('uknots');
k3 = t('uknots');
k = unique([k1 k2 k3]);
n = length(k)
kua = [];
kub = [];
kuc = [];
for i = 1:n
  i1 = length(find(k1 == k(i)));
  i2 = length(find(k2 == k(i)));
  i3 = length(find(k3 == k(i)));
  m = max(i1, i2, i3);
  kua = [kua k(i)*ones(1,m-i1)];  
  kub = [kub k(i)*ones(1,m-i2)];
  kuc = [kuc k(i)*ones(1,m-i3)];
end  

// V knots
k1 = r1('vknots');
k2 = r2('vknots');
k3 = t('vknots');
k = unique([k1 k2 k3]);
n = length(k)
kva = [];
kvb = [];
kvc = [];
for i = 1:n
  i1 = length(find(k1 == k(i)));
  i2 = length(find(k2 == k(i)));
  i3 = length(find(k3 == k(i)));
  m = max(i1, i2, i3);
  kva = [kva k(i)*ones(1,m-i1)];  
  kvb = [kvb k(i)*ones(1,m-i2)];
  kvc = [kvc k(i)*ones(1,m-i3)];
end  

r1 = nrbkntins(r1, kua, kva);
r2 = nrbkntins(r2, kub, kvb);
t  = nrbkntins(t,  kuc, kvc);

// combine coefficients to construct Coons surface
coefs = hypermat(size(t('cntrl')));

// bug => should be able to do r1('cntrl')(1,:,:) not the code below
r1cntrl = r1('cntrl');
r2cntrl = r2('cntrl');
tcntrl = t('cntrl');

coefs(1,:,:) = r1cntrl(1,:,:) + r2cntrl(1,:,:) - tcntrl(1,:,:);
coefs(2,:,:) = r1cntrl(2,:,:) + r2cntrl(2,:,:) - tcntrl(2,:,:);
coefs(3,:,:) = r1cntrl(3,:,:) + r2cntrl(3,:,:) - tcntrl(3,:,:);
coefs(4,:,:) = r1cntrl(4,:,:) + r2cntrl(4,:,:) - tcntrl(4,:,:);
srf = nrbmak(coefs, r1('uknots'), r1('vknots'));

endfunction
