function inurb = nrbdegelev(nurb, utimes, vtimes)
// Degree elevate the NURB.
//
//          inurb = nrbdegelev(nurb,utimes[,vtimes]);
//
// INPUT:
//
//   nurb   - NURB structure (see nrbmak.m).
//   utimes - degree elevate utimes along u direction.
//   vtimes - degree elevate vtimes along v direction.
//
// OUTPUT:
//
//   inurb  - new NURB structure with knots inserted.
//

[nlhs,nrhs] = argn(0);

if nrhs < 2
  error('Input argument must include the NURB and degree increment.');
end

if nurb('vknots') == []
  // NURB represents a curve
  if utimes == 0
    inurb = nurb;
  else
    [coefs,knots] = bspdegelev(nurb(2),nurb(3),nurb(4),utimes);
    inurb = nrbmak(coefs,knots);
  end  
else  
  // NURB represents a surface
  [dim,num1,num2] = size(nurb(3));

  // Degree elevate along the v direction
  if vtimes == 0
    coefs = nurb(3);
    vknots = nurb(5);
  else
    coefs = matrix(nurb(3),4*num1,num2);
    [coefs,vknots] = bspdegelev(nurb(2)(2),coefs,nurb(5),vtimes);
    num2 = size(coefs,2);
    coefs = matrix(coefs,[4 num1 num2]);
  end

  // Degree elevate along the u direction
  if utimes == 0
    uknots = nurb(4);
  else  
    // no scilab equivalent for val = permute(val,[1 3 2]);
    temp = hypermat([4 num2 num1]);
    temp(1,:,:) = matrix(coefs(1,:,:),[num1 num2])';
    temp(2,:,:) = matrix(coefs(2,:,:),[num1 num2])';
    temp(3,:,:) = matrix(coefs(3,:,:),[num1 num2])';
    temp(4,:,:) = matrix(coefs(4,:,:),[num1 num2])';

    tcoefs = matrix(temp,4*num2,num1);
    [tcoefs,uknots] = bspdegelev(nurb(2)(1),tcoefs,nurb(4),utimes);
    nt = size(tcoefs,2);
    tcoefs = matrix(tcoefs,[4 num2 nt]);
    
    // no scilab equivalent for val = permute(val,[1 3 2]);
    coefs = hypermat([4 nt num2]);
    coefs(1,:,:) = matrix(tcoefs(1,:,:),[num2 nt])';
    coefs(2,:,:) = matrix(tcoefs(2,:,:),[num2 nt])';
    coefs(3,:,:) = matrix(tcoefs(3,:,:),[num2 nt])';
    coefs(4,:,:) = matrix(tcoefs(4,:,:),[num2 nt])';
  end 
  
  // construct new NURB
  if utimes > 0 | vtimes > 0
    inurb = nrbmak(coefs, uknots, vknots);
  else
    inurb = nurb;
  end  
end
endfunction






