function ders = nrbderiv(nurb)
// Construct the first and second derivative representations of a
// NURBS curve or surface.
//
//        ders = nrbderiv(nurb);
//
// Inputs:
//
//   nurb   - NURB curve or surface
//
// Outputs:
//
//   ders   - a type list of bspline curve and surface derivatives.
//
// NOTE:
//
// I'm not 100% happy with this function and nrbdegelev, and they will
// probably be changed in the near future. Also the second derivatives
// for NURBS surfaces are not being evaluated yet!

[nlhs,nrhs] = argn(0);

if nurb('vknots') == []
  
  // NURB structure represents a curve
  ders = tlist(['DNURBS','cu','cuu']);
  
  // first derivative along U
  [dcoefs,dknots] = bspderiv(nurb('cntrl'),nurb('uknots'));
  ders('cu') = nrbmak(dcoefs,dknots);

  // second derivative along U
  [dcoefs,dknots] = bspderiv(dcoefs,dknots);
  ders('cuu') = nrbmak(dcoefs,dknots);
  
else
  
  // NURB structure represents a surface
  ders = tlist(['DNURBS','su','sv','suu','suv','svu','svv']);
  
  // taking first derivatives along the u direction
  cntrl = nurb('cntrl');
  [dim,num1,num2] = size(cntrl);
  temp = hypermat([4 num2 num1]);
  temp(1,:,:) = matrix(cntrl(1,:,:),[num1 num2])';
  temp(2,:,:) = matrix(cntrl(2,:,:),[num1 num2])';
  temp(3,:,:) = matrix(cntrl(3,:,:),[num1 num2])';
  temp(4,:,:) = matrix(cntrl(4,:,:),[num1 num2])';
  dc = matrix(temp,4*num2,num1);
  [dc,dku] = bspderiv(dc,nurb('uknots'));
  nt = size(dc,2);
  temp = matrix(dc,[4 num2 nt]);
  dcu = hypermat([4 nt num2]);
  dcu(1,:,:) = matrix(temp(1,:,:),[num2 nt])';
  dcu(2,:,:) = matrix(temp(2,:,:),[num2 nt])';
  dcu(3,:,:) = matrix(temp(3,:,:),[num2 nt])';
  dcu(4,:,:) = matrix(temp(4,:,:),[num2 nt])';
  ders('su') = nrbmak(dcu,dku,nurb('vknots'));
  
  // taking first derivatives along the v direction
  dc = matrix(nurb('cntrl'),4*num1,num2);
  [dc,dkv] = bspderiv(dc,nurb('vknots'));
  num2 = size(dc,2);
  dcv = matrix(dc,[4 num1 num2]);
  ders('sv') = nrbmak(dcv,nurb('uknots'),dkv);

  // taking second derivatives along the u direction
  [dim,num1,num2] = size(dcu);
  temp = hypermat([4 num2 num1]);
  temp(1,:,:) = matrix(dcu(1,:,:),[num1 num2])';
  temp(2,:,:) = matrix(dcu(2,:,:),[num1 num2])';
  temp(3,:,:) = matrix(dcu(3,:,:),[num1 num2])';
  temp(4,:,:) = matrix(dcu(4,:,:),[num1 num2])';
  dc = matrix(temp,4*num2,num1);
  [dc,dkuu] = bspderiv(dc,dku);
  nt = size(dc,2);
  temp = matrix(dc,[4 num2 nt]);
  dcuu = hypermat([4 nt num2]);
  dcuu(1,:,:) = matrix(temp(1,:,:),[num2 nt])';
  dcuu(2,:,:) = matrix(temp(2,:,:),[num2 nt])';
  dcuu(3,:,:) = matrix(temp(3,:,:),[num2 nt])';
  dcuu(4,:,:) = matrix(temp(4,:,:),[num2 nt])';
  ders('suu') = nrbmak(dcuu,dkuu,nurb('vknots'));

  [dim,num1,num2] = size(dcv);
  temp = hypermat([4 num2 num1]);
  temp(1,:,:) = matrix(dcv(1,:,:),[num1 num2])';
  temp(2,:,:) = matrix(dcv(2,:,:),[num1 num2])';
  temp(3,:,:) = matrix(dcv(3,:,:),[num1 num2])';
  temp(4,:,:) = matrix(dcv(4,:,:),[num1 num2])';
  dc = matrix(temp,4*num2,num1);
  [dc,dkuv] = bspderiv(dc,dku);
  nt = size(dc,2);
  temp = matrix(dc,[4 num2 nt]);
  dcuv = hypermat([4 nt num2]);
  dcuv(1,:,:) = matrix(temp(1,:,:),[num2 nt])';
  dcuv(2,:,:) = matrix(temp(2,:,:),[num2 nt])';
  dcuv(3,:,:) = matrix(temp(3,:,:),[num2 nt])';
  dcuv(4,:,:) = matrix(temp(4,:,:),[num2 nt])';
  ders('suv') = nrbmak(dcuv,dkuv,dkv);
  
  // taking second derivatives along the v direction
  [dim,num1,num2] = size(dcv);
  dc = matrix(dcv,4*num1,num2);
  [dc,dkvv] = bspderiv(dc,dkv);
  num2 = size(dc,2);
  dcvv = matrix(dc,[4 num1 num2]);
  ders('svv') = nrbmak(dcvv,nurb('uknots'),dkvv);

  [dim,num1,num2] = size(dcu);
  dc = matrix(dcu,4*num1,num2);
  [dc,dkvu] = bspderiv(dc,dkv);
  num2 = size(dc,2);
  dcvu = matrix(dc,[4 num1 num2]);
  ders('svu') = nrbmak(dcvu,dku,dkvu);
  
end

function [coefs,knots] = bspderiv(coefs,knots)
// Differentiate a univariate B-spline.
//
// INPUT:
//
//   coefs   - control points of the spline
//   knots   - knot sequence 
//
// OUTPUT:
//
//   coefs  - control points of the spline derivative
//   knots  - knot sequence of the spline derivative
//

[dim,nc] = size(coefs);   // dimension and number of control points
deg = length(knots)-nc-1; // order of the spline

if deg <= 0
  // All derivatives of an order larger than the spline degree are zero.
  coefs = zeros(size(coefs,1),1);
  knots = [0.0 1.0];
else
  tt = knots(deg+1+[0:nc])-knots(1:nc+1);
  z = find(tt > 0);
  nn = length(z);
  temp1 = (mtlb_diff([zeros(1,dim);coefs.';zeros(1,dim)],1)).';
  temp2 = tt(z)/deg;
  coefs = temp1(:,z)./temp2(ones(1,dim),:);
  knots = [knots(z) knots(nc+2:nc+deg+1)];
end


 
