function [p,w] = nrbeval(nurb,ut,vt)
// Evaluate a NURBS at parametric points.
//
//          [p,w] = nrbeval(nurb,ut,vt)
//
// INPUTS:
//
//   nurb - NURB curve or surface.
//   ut   - parametric evaluation points along u.
//   vt   - paramteric evaluation points along v.
//
// OUTPUTS:
//
//   p    - evaluated (x,y,z) coordinates.
//   w    - evaluated weights w.
//
// NOTES:
//
//   If the weights are requested the algorithm returns the homogeneous
//   coordinates in p = (w*x,w*y,w*z) otherwise it returns the 3D
//   cartesian coordinates p = (x,y,z).
//

[nlhs,nrhs] = argn(0);

if nrhs < 2 
  error('Incorrect number of input arguments');
end

foption = 1;    // output format 3D cartesian coordinates
if nlhs == 2
  foption = 0;  // output format 4D homogeneous coordinates 
end
   
if type(nurb) ~= 16
  error('NURB representation is not a type list!');
end

if nurb('vknots') == []
  // NURB structure represents a curve
  //  tt represent a vector of parametric points in the u direction

  [deg,cntrl,uknots] = nurb(2:4)

  val = bspeval(deg,cntrl,uknots,ut);   

  w = val(4,:);
  p = val(1:3,:);
  if foption
    p = p./w(ones(1,3),:);
  end

else

  // NURB structure represents a surface

  [deg,cntrl,uknots,vknots] = nurb(2:5);

  // number of control points
  [dim,num1,num2] = size(cntrl);

  if nrhs == 3 
    // Evaluate over a [u,v] grid
    // ut represents the u direction
    // vt represents the v direction

    nt1 = length(ut);
    nt2 = length(vt);

    // Evaluate along the v direction
    val = matrix(cntrl,4*num1,num2);
    val = bspeval(deg(2),val,vknots,vt);
    val = matrix(val,[4 num1 nt2]);
    
    // Evaluate along the u direction

    // no scilab equivalent for val = permute(val,[1 3 2]);
    val2 = hypermat([4 nt2 num1]);
    val2(1,:,:) = matrix(val(1,:,:),[num1 nt2])';
    val2(2,:,:) = matrix(val(2,:,:),[num1 nt2])';
    val2(3,:,:) = matrix(val(3,:,:),[num1 nt2])';
    val2(4,:,:) = matrix(val(4,:,:),[num1 nt2])';

    val = matrix(val2,4*nt2,num1);
    val = bspeval(deg(1),val,uknots,ut);
    val = matrix(val,[4 nt2 nt1]);

    // no scilab equivalent for val = permute(val,[1 3 2]);
    val2 = hypermat([4 nt1 nt2]);
    val2(1,:,:) = matrix(val(1,:,:),[nt2 nt1])';
    val2(2,:,:) = matrix(val(2,:,:),[nt2 nt1])';
    val2(3,:,:) = matrix(val(3,:,:),[nt2 nt1])';
    val2(4,:,:) = matrix(val(4,:,:),[nt2 nt1])';

    w = val2(4,:,:);
    p = val2(1:3,:,:);
    if foption 
      p = p./w(ones(1,3),:,:);
    end

  else

    // Evaluate at scattered points
    // ut(1,:) represents the u direction
    // ut(2,:) represents the v direction

    nt = size(tt,2);

    // Evaluate along the v direction
    uval = matrix(cntrl,4*num1,num2);
    uval = bspeval(deg(2),uval,uknots,ut(2,:));
    uval = matrix(val,[4 num1 nt]);

    // Evaluate along the u direction
    val = zeros(4,nt);
    for v = 1:nt
      cntrl = matrix(uval(:,:,v),[4 num1]);
      val(:,v) = bspeval(deg(1),cntrl,vknots,ut(1,v));
    end

    w = val(4,:);
    p = val(1:3,:);
    if foption 
       p = p./w;
    end

  end

end
endfunction

