function inurb = nrbkntins(nurb,iuknots,ivknots)
// Insert new knots into the NURB.
//
//          inurb = nrbkntins(nurb,iuknots[,ivknots]);
//
// Examples:
//
//   1. Insert two knots into a curve, one at 0.3 and another
//      twice at 0.4
//
//          nurb = nrbkntins(curve,[0.3 0.4 0.4])
//
//   2. Insert into a surface two knots as (1) into the U knot
//      sequence and one knot into the V knot sequence at 0.5.
//
//          nurb = nrbkntins(srf,[0.3 0.4 0.4],[0.5])
//
// INPUT:
//
//   nurb    - NURB structure.
//   iuknots - knots to be inserted along u direction.
//   ivknots - knots to be inserted along v direction.
//
// OUTPUT:
//
//   inurb  - new NURB structure with knots inserted.
//
// NOTE:
//
//   No knot multiplicity will be increased beyond the order of the spline.
//

[nlhs,nrhs] = argn(0);

if nrhs < 2
  error('Input argument must include the NURB and knots to be inserted');
end

if length(nurb('degree')) == 2 & nrhs < 3
  error('Incorrect number of input arguments');
end  

if nurb('vknots') == []
  // NURB represents a curve
  if iuknots == []
    inurb = nurb;
  else
    [coefs,knots] = bspkntins(nurb(2),nurb(3),nurb(4),gsort(iuknots,'g','i'));
    inurb = nrbmak(coefs,knots);
  end
else  
  // NURB represents a surface
  [dim,num1,num2] = size(nurb(3));

  // Insert knots along the v direction
  if ivknots == []
    coefs = nurb(3);
    vknots = nurb(5);
  else
    coefs = matrix(nurb(3),4*num1,num2);
    [coefs,vknots] = bspkntins(nurb(2)(2),coefs,nurb(5),gsort(ivknots,'g','i'));
    num2 = size(coefs,2);
    coefs = matrix(coefs,[4 num1 num2]);
  end

  // Insert knots along the u direction
  if iuknots == []
    uknots = nurb(4);
  else  
    // no scilab equivalent for val = permute(val,[1 3 2]);
    temp = hypermat([4 num2 num1]);
    temp(1,:,:) = matrix(coefs(1,:,:),[num1 num2])';
    temp(2,:,:) = matrix(coefs(2,:,:),[num1 num2])';
    temp(3,:,:) = matrix(coefs(3,:,:),[num1 num2])';
    temp(4,:,:) = matrix(coefs(4,:,:),[num1 num2])';

    tcoefs = matrix(temp,4*num2,num1);
    [tcoefs,uknots] = bspkntins(nurb(2)(1),tcoefs,nurb(4),gsort(iuknots,'g','i'));
    nt = size(tcoefs,2);
    tcoefs = matrix(tcoefs,[4 num2 nt]);
    
    // no scilab equivalent for val = permute(val,[1 3 2]);
    coefs = hypermat([4 nt num2]);
    coefs(1,:,:) = matrix(tcoefs(1,:,:),[num2 nt])';
    coefs(2,:,:) = matrix(tcoefs(2,:,:),[num2 nt])';
    coefs(3,:,:) = matrix(tcoefs(3,:,:),[num2 nt])';
    coefs(4,:,:) = matrix(tcoefs(4,:,:),[num2 nt])';
  end 
  
  // construct new NURB
  if iuknots == [] & ivknots == []
    inurb = nurb;
  else  
    inurb = nrbmak(coefs,uknots,vknots);
  end  
end
endfunction






