function nurb = nrbmak(cntrl,uknots,vknots)
// Construct a NURB curve or surface structure, and check the format.
// 
// The NURB curve or surface is represented by a 4 dimensional
// b-spline.
//
// INPUT:
//
//   cntrl  - Control points, homogeneous coordinates (wx,wy,wz,w) 
//
//            For a curve   [dim,nu]   matrix
//            For a surface [dim,nu,nv] hypermatrix
//            where nu  is along the u direction and
//                  nv  is along the v direction.
//                  dim is the dimension valid options are:
//                  2 .... (x,y)        2D cartesian coordinates
//                  3 .... (x,y,z)      3D cartesian coordinates   
//                  4 .... (wx,wy,wz,w) 4D homogeneous coordinates
//
//   uknots - Knot sequence along the parametric u direction.
//   vknots - Knot sequence along the paramteric v direction.
//
// OUTPUT:
//
//   nurb   - A type list defining the NURB.            
//            (1) ... type list
//            (2) ... spline degree
//            (3) ... control points
//            (4) ... knot sequence along the u direction
//            (5) ... knot sequence along the v direction
//   
// NOTES:
//
//   Its assumed that the input knot sequences span the
//   interval [0.0,1.0] and are clamped to the control
//   points at the end by a knot multiplicity equal to
//   the spline order.

[nlhs,nrhs] = argn(0);

if nrhs < 2 
  error('Invalid number of input arguments');
end

// The NURB structure is usually accessed via these field names.
nurb = tlist(['NURBS','degree','cntrl','uknots','vknots']);

// Force the u knot sequence to be a vector in ascending order,
// and normalise between [0.0,1.0]
nku = prod(size(uknots));
uknots = matrix(gsort(uknots,'g','i'),1,nku);
uknots = (uknots-uknots(1))/(uknots($)-uknots(1));
if uknots(1) == uknots($)
   error('The extreme knots for u should be different.')
end  
nurb('uknots') = uknots;

if nrhs < 3 
  // Construct a NURB curve

  // Spline control points 
  [dim,nu] = size(cntrl);
  if dim < 2 | dim > 4 
    error('Illegal control point format');
  elseif dim < 4 
    nurb('cntrl') = [zeros(3,nu);ones(1,nu)];
    nurb('cntrl')(1:dim,:) = cntrl;
  else
    nurb('cntrl') = cntrl;
  end

  // v knot sequence is empty
  nurb('vknots') = [];

  // Spline Degree
  nurb('degree') = nku-nu-1;  
  if nurb('degree') < 0 
    error('Spline order must always be a positive integer');
  end
  
else

  // Construct a NURB surface

  // Spline control points 
  [dim,nu,nv] = size(cntrl);
  if dim < 2 | dim > 4 
    error('Illegal control point format');
  elseif dim < 4 
    // *** bug *** nurb('cntrl') = hypermat([4 nu nv]);
    //  here -->   nurb('cntrl')(1:dim,:,:) = cntrl;
    //  nurb('cntrl')(4,:,:) = ones(nu,nv);

    // alternative
    a = hypermat([4 nu nv]);
    a(1:dim,:,:) = cntrl;
    a(4,:,:) = ones(nu,nv);
    nurb('cntrl') = a;

  else  
    nurb('cntrl') = cntrl;
  end

  // Force the v knot sequence to be a vector in ascending order
  nkv = prod(size(vknots));
  vknots = matrix(-sort(-vknots),1,nkv);
  vknots = (vknots-vknots(1))/(vknots($)-vknots(1));
  if vknots(1) == vknots($)
    error('The extreme knots for v should be different.')
  end  
  nurb('vknots') = vknots;

  // Spline Degree
  nurb('degree') = [nku-nu-1, nkv-nv-1];

  if nurb('degree')(1) < 0 | nurb('degree')(2) < 0 
    error('Spline degree must always be a positive integer');
  end
end
endfunction
