function srf = nrbrevolve(curve,pnt,vec,theta)
// Construct a surface by revolving the profile curve
// around an axis defined by a point and a unit vector.
//
//    srf = nrbrevolve(curve,point,vector[,theta])
//
// INPUT:
//
//   curve  - NURB profile curve.
//   point  - coordinate of the point.
//   vector - rotation axis.
//   theta  - angle to revolve curve (default: 2*pi).
//
// OUTPUT:
//
//   srf   - The resulting revolved NURB surface.
//
// NOTE:
//
//   The algorithm:
//
//     1) translate the point to the origin (0,0,0)
//     2) rotate the vector into alignment with the z-axis
//
//     for each control point along the curve
//
//     3) determine the radius and angle of control
//        point to the z-axis
//     4) construct a circular arc in the x-y plane with 
//        this radius and start angle and sweep angle theta 
//     5) combine the arc and profile, coefs and weights.
//  
//     next control point
//
//     6) rotate and translate the surface back into position
//        by reversing 1 and 2.
//

[nlhs,nrhs]=argn(0);

if nrhs < 3
  error('Not enough arguments to construct revolved surface');
end

if curve('vknots') ~= []
  error('NURB to rotate must be a 2D/3D curve');
end
 
// default full rotation
if nrhs < 4
  theta = 2.0*%pi;
end

// default translate curve the center point to the origin
if pnt == []
  pnt = zeros(3,1);
end

if length(pnt) ~= 3
  error('All point and vector coordinates must have 3D corrdinates.');
end

// Translate and rotate the curve into alignment with the z-axis
T  = vectrans(-pnt);
if vec(1) == 0
  angx = 0;
else  
  angx = vecangle(vec(1),vec(3));
end
RY = vecroty(-angx);
vectmp = RY*[vecnorm(vec(:));1.0];
if vectmp(2) == 0
  angy = 0;
else  
  angy = vecangle(vectmp(2),vectmp(3));
end
RX = vecrotx(angy);
curve = nrbtform(curve,RX*RY*T);

// Construct an circular arc 
arc = nrbcirc(1.0,[],0.0,theta);

// Construct the surface
narc = size(arc(3),2);
ncurve = size(curve(3),2);
coefs = hypermat([4 narc ncurve]);
angle = vecangle(curve(3)(2,:),curve(3)(1,:));
radius = vecmag(curve(3)(1:2,:));
for i = 1:ncurve  
  coefs(:,:,i) = vecrotz(angle(i))*vectrans([0.0 0.0 curve(3)(3,i)])*...
          vecscale([radius(i) radius(i)])*arc(3);
  coefs(4,:,i) = coefs(4,:,i)*curve(3)(4,i);
end
srf = nrbmak(coefs,arc(4),curve(4));

// Rotate and translate the surface back into position
T = vectrans(pnt);
RX = vecrotx(-angy);
RY = vecroty(angx);
srf = nrbtform(srf,T*RY*RX);  
endfunction



