function srf = nrbruled(crv1, crv2)
// Constructs a ruled surface between two NURBS curves.
//
// INPUT:
//
//   crv1 - first  NURBS curve
//   crv2 - second NURBS curve
//
// OUTPUT:
//
//   srf  - resulting ruled surface
//
// NOTE:
//
//   The ruled surface is ruled along the V direction. 
//

if (crv1('vknots') ~= []) | (crv2('vknots') ~=[])
  error('nurb curves must both be along U direction');
end

// ensure both curves have a common degree
d = max(crv1('degree'), crv2('degree'));
crv1 = nrbdegelev(crv1, d - crv1('degree'));
crv2 = nrbdegelev(crv2, d - crv2('degree'));

// merge the knot vectors, to obtain a common knot vector
k1 = crv1('uknots');
k2 = crv2('uknots');
ku = unique([k1 k2]);
n = length(ku)
ka = [];
kb = [];
for i = 1:n
  i1 = length(find(k1 == ku(i)));
  i2 = length(find(k2 == ku(i)));
  m = max(i1, i2);
  ka = [ka ku(i)*ones(1,m-i1)];
  kb = [kb ku(i)*ones(1,m-i2)];
end  
crv1 = nrbkntins(crv1, ka);
crv2 = nrbkntins(crv2, kb);

coefs = hypermat([4 size(crv1('cntrl'),2) 2]);
coefs(:,:,1) = crv1('cntrl');
coefs(:,:,2) = crv2('cntrl');
srf = nrbmak(coefs, crv1('uknots'), [0 0 1 1])
endfunction
