// scilab interface to nurb toolbox c functions
//

#include <stack-c.h>
#include "bspline.h"

// ========================= BINCOEFFI ===================================
// Scilab interface for the C function bincoeff
//
// SCILAB SYNTAX:
//
//        [c] = bincoeff(n,k)
// 
// INPUT:
//
//     ( n )      n!          n integer
//     (   ) = --------   
//     ( k )   k!(n-k)!       k integer
//
// OUTPUT:
//
//   c - coefficient          double
//
int bincoeffi(char* fname, unsigned long fname_len)
{
  int ierr;
  int n, mn, nn;
  int k, mk, nk;
  int c;

  CheckLhs(1,1);
  CheckRhs(2,2);

  // Extract the stack index of the input arguments 
  GetRhsVar(1, "i", &mn, &nn, &n); 
  GetRhsVar(2, "i", &mk, &nk, &k); 

  if (mn != 1 | nn != 1)
  {
    sciprint("Internal error in bincoeff: n is not an integer value");
    Error(999);
    return (0);
  }

  // check row dimensions
  if (mk != 1 | mk != 1)
  {
    sciprint("Internal error in bincoeff: k is not an integer value");
    Error(999);
    return (0);
  }

  // Create return variables
  CreateVar(3, "d", &mn, &nn, &c);

  *stk(c) = bincoeff(*istk(n), *istk(k));

  LhsVar(1) = 3;

  PutLhsVar();

  return (0);
}

// ========================= BSPEVALI ===================================
// Scilab interface for the C function bspeval
//
// SCILAB SYNTAX:
//
//        p = bspeval(d,c,k,u)
// 
// INPUT:
//
//   d - degree of the B-Spline  integer
//   c - control points          double  matrix(mc,nc)
//   k - knot sequence           double  vector(nk)
//   u - parametric points       double  vector(nu)
//
// OUTPUT:
//
//   p - evaluated points                 double matrix(mc,nu)
//
//
int bspevali(char* fname, unsigned long fname_len)
{
  int ierr;
  int d, md, nd;
  int c, mc, nc;
  int k, mk, nk;
  int u, mu, nu;
  int p;

  CheckLhs(1,1);
  CheckRhs(4,4);

  // Extract the stack index of the input arguments 
  GetRhsVar(1, "i", &md, &nd, &d); 
  GetRhsVar(2, "d", &mc, &nc, &c); 
  GetRhsVar(3, "d", &mk, &nk, &k); 
  GetRhsVar(4, "d", &mu, &nu, &u); 

  if (md != 1 & nd != 1)
  {
    sciprint("Internal error in bspevali: degree is integer value");
    Error(999);
    return (0);
  }

  // check row dimensions
  if (mk != 1 | mu != 1)
  {
    sciprint("Internal error in bspevali: invalid matrix rows");
    Error(999);
    return (0);
  }

  // Create return variables
  CreateVar(5, "d", &mc, &nu, &p);

  ierr = bspeval(*istk(d), stk(c), mc, nc, stk(k), nk, stk(u), nu, stk(p));

  LhsVar(1) = 5;

  PutLhsVar();

  return (0);
}

// ========================= BSPDEVALI ===================================
// Scilab interface for the C function bspdeval
//
// SCILAB SYNTAX:
//
//        [v] = bspdeval(d,c,k,u,n)
// 
// INPUT:
//
//   d - degree of the B-Spline  integer
//   c - control points          double  matrix(mc,nc)
//   k - knot sequence           double  vector(nk)
//   u - parametric point        double
//   n - nth derivative          integer
//
// OUTPUT:
//
//   p - evaluated point and derivatives double matrix(mc,n)
//
int bspdevali(char* fname, unsigned long fname_len)
{
  int ierr;
  int d, md, nd;
  int c, mc, nc;
  int k, mk, nk;
  int u, mu, nu;
  int n, mn, nn;
  int p, np;

  CheckLhs(1,1);
  CheckRhs(5,5);

  // Extract the stack index of the input arguments 
  GetRhsVar(1, "i", &md, &nd, &d); 
  GetRhsVar(2, "d", &mc, &nc, &c); 
  GetRhsVar(3, "d", &mk, &nk, &k); 
  GetRhsVar(4, "d", &mu, &nu, &u); 
  GetRhsVar(5, "i", &mn, &nn, &n); 

  if (md != 1 | nd != 1)
  {
    sciprint("Internal error in bspdevali: degree is integer value");
    Error(999);
    return (0);
  }

  // check row dimensions
  if (mk != 1 | mu != 1)
  {
    sciprint("Internal error in bspdevali: invalid matrix rows");
    Error(999);
    return (0);
  }

  if (mn != 1 | nn != 1)
  {
    sciprint("Internal error in bspdevali: integer value");
    Error(999);
    return (0);
  }

  // Create return variables
  np = *istk(n)+1;
  CreateVar(6, "d", &mc, &np, &p);

  ierr = bspdeval(*istk(d), stk(c), mc, nc, stk(k), nk, *stk(u), *istk(n),
                  stk(p));

  LhsVar(1) = 6;

  PutLhsVar();

  return (0);
}

// ========================= BSPKNTINSI ===================================
// Scilab interface for the C function bspkntins
//
// SCILAB SYNTAX:
//
//        [ic,ik] = bspkntins(d,c,k,u)
// 
// INPUT:
//
//   d - degree of the B-Spline  integer
//   c - control points          double  matrix(mc,nc)
//   k - knot sequence           double  vector(nk)
//   u - knots to be inserted    double  vector(nu)
//
// OUTPUT:
//
//   ic - new control points     double matrix(mc,nc+nu)
//   ik - new knot sequence      double vector(nk+nu)
//
int bspkntinsi(char* fname, unsigned long fname_len)
{
  int ierr;
  int d, md, nd;
  int c, mc, nc;
  int k, mk, nk;
  int u, mu, nu;
  int ic, nic;
  int ik, nik;

  CheckLhs(2,2);
  CheckRhs(4,4);

  // Extract the stack index of the input arguments 
  GetRhsVar(1, "i", &md, &nd, &d); 
  GetRhsVar(2, "d", &mc, &nc, &c); 
  GetRhsVar(3, "d", &mk, &nk, &k); 
  GetRhsVar(4, "d", &mu, &nu, &u); 

  if (md != 1 & nd != 1)
  {
    sciprint("Internal error in bspkntinsi: degree is integer value");
    Error(999);
    return (0);
  }

  // check row dimensions
  if (mk != 1 | mu != 1)
  {
    sciprint("Internal error in bspkntinsi: invalid matrix rows");
    Error(999);
    return (0);
  }

  // Create return variables
  nic = nc + nu; 
  nik = nk + nu; 
  CreateVar(5, "d", &mc, &nic, &ic);
  CreateVar(6, "d", &mk, &nik, &ik);

  ierr = bspkntins(*istk(d), stk(c), mc, nc, stk(k), nk, stk(u), nu,
                   stk(ic), stk(ik));

  LhsVar(1) = 5;
  LhsVar(2) = 6;

  PutLhsVar();

  return (0);
}

// ========================= BSPDEGELEV ===================================
// Scilab interface for the C function bspdegelev
//
// SCILAB SYNTAX:
//
//        [ic,ik] = bspdegelev(d,c,k,t)
// 
// INPUT:
//
//   d - degree of the B-Spline   integer
//   c - control points           double  matrix(mc,nc)
//   k - knot sequence            double  vector(nk)
//   t - raise degree t times     integer
//
// OUTPUT:
// 
//   ic - new control points      double matrix(mc,nh)
//   ik - new knot sequence       double vector(nh+d+1)
//
//   where nh is the new number of control points
//
int bspdegelevi(char* fname, unsigned long fname_len)
{
  int ierr;
  int d, md, nd;
  int c, mc, nc;
  int k, mk, nk;
  int t, mt, nt;
  int ic, nic;
  int ik, nik;
  int deg, tt;

  int i, j, n, nh;
  double *inc, *ink, *icc, *ikk;

  CheckLhs(2,2);
  CheckRhs(4,4);

  // Extract the stack index of the input arguments 
  GetRhsVar(1, "i", &md, &nd, &d); 
  GetRhsVar(2, "d", &mc, &nc, &c); 
  GetRhsVar(3, "d", &mk, &nk, &k); 
  GetRhsVar(4, "i", &mt, &nt, &t); 

  if (md != 1 & nd != 1)
  {
    sciprint("Internal error in bspkntinsi: degree is integer value");
    Error(999);
    return (0);
  }

  // check row dimensions
  if (mk != 1 | mt != 1)
  {
    sciprint("Internal error in bspkntinsi: invalid matrix rows");
    Error(999);
    return (0);
  }

  // allocate work space t times larger than original number
  // of control points and knots
  tt = *istk(t);
  inc = (double *) malloc((tt+1)*mc*nc*sizeof(double));
  ink = (double *) malloc((tt+1)*nk*sizeof(double));

  deg = *istk(d);
  ierr = bspdegelev(deg, stk(c), mc, nc, stk(k), nk, tt, &nh, inc, ink);

  // Create return variables
  nic = nh + 1;
  nik = nic + deg + tt + 1;
  CreateVar(5, "d", &mc, &nic, &ic);
  CreateVar(6, "d", &mk, &nik, &ik);

  // copy new control points
  icc = stk(ic);
  n = mc * nic;
  for (i = 0; i < n; i++)
    icc[i] = inc[i];   
  
  // copy new knots
  ikk = stk(ik);
  for (i = 0; i < nik; i++)
    ikk[i] = ink[i];      

  LhsVar(1) = 5;
  LhsVar(2) = 6;

  PutLhsVar();

  // free working space
  free(ink);
  free(inc);

  return (0);
}

static TabF Tab[] =
{ 
  {bincoeffi,   "bincoeff"},
  {bspevali,    "bspeval"},
  {bspdevali,   "bspdeval"},
  {bspkntinsi,  "bspkntins"},
  {bspdegelevi, "bspdegelev"}
}; 

int C2F(nurbslib)(void)
{
  Rhs = Max(0, Rhs);
  (*(Tab[Fin-1].f))(Tab[Fin-1].name,strlen(Tab[Fin-1].name));
  return 0;
} 




